"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.parseStack = exports.captureErrorString = exports.captureError = exports.captureString = exports.at = exports.capture = exports.getFilterIgnoredPackages = exports.setFilterIgnoredPackages = exports.getIgnoredPackages = exports.removeIgnoredPackage = exports.addIgnoredPackage = exports.setFilterNodeInternals = exports.getFilterNodeInternals = exports.getCwd = exports.setCwd = exports.CallSiteLike = void 0;
const module_1 = require("module");
const call_site_like_js_1 = require("./call-site-like.js");
var call_site_like_js_2 = require("./call-site-like.js");
Object.defineProperty(exports, "CallSiteLike", { enumerable: true, get: function () { return call_site_like_js_2.CallSiteLike; } });
const regExpEscape = (s) => s.replace(/[-[\]{}()*+?.,\\^$|#\s]/g, '\\$&');
// platform portability accommodation
/* c8 ignore start */
let cwd = typeof process === 'object' &&
    process &&
    typeof process.cwd === 'function'
    ? process.cwd()
    : undefined;
/* c8 ignore stop */
/**
 * Set the effective cwd for shortening filenames in stack traces
 * Set to `undefined` to show full absolute paths.
 */
const setCwd = (c) => (cwd = c);
exports.setCwd = setCwd;
/**
 * Get the effective cwd for shortening filenames in stack traces
 * If set to `undefined`, then will show full absolute paths.
 */
const getCwd = () => cwd;
exports.getCwd = getCwd;
let filterNodeInternals = true;
/**
 * Get the current value indicating whether node internals should be
 * filtered out. (Defaults to true)
 */
const getFilterNodeInternals = () => filterNodeInternals;
exports.getFilterNodeInternals = getFilterNodeInternals;
/**
 * Set whether node internals should be filtered out.
 */
const setFilterNodeInternals = (s) => (filterNodeInternals = s);
exports.setFilterNodeInternals = setFilterNodeInternals;
// these are packages that function somewhat like node internals,
// for tap's purposes, and @tapjs packages themselves, when loaded
// from node_modules.
const ignoredPackages = [
    '@tapjs',
    'ts-node',
    'pirates',
    'function-loop',
    '@cspotcode/source-map-support',
    'signal-exit',
    'async-hook-domain',
];
let dirty = false;
/**
 * Add a package name to the list of deps that should be excluded
 * from stack traces.
 */
const addIgnoredPackage = (s) => {
    const i = ignoredPackages.indexOf(s);
    if (i === -1) {
        ignoredPackages.push(s);
        dirty = true;
    }
};
exports.addIgnoredPackage = addIgnoredPackage;
/**
 * Remove a package name from the list of deps that should be excluded
 * from stack traces.
 */
const removeIgnoredPackage = (s) => {
    const i = ignoredPackages.indexOf(s);
    if (i !== -1) {
        ignoredPackages.splice(i, 1);
        dirty = true;
    }
};
exports.removeIgnoredPackage = removeIgnoredPackage;
/**
 * Get a read-only copy of the list of deps that should be excluded
 * from stack traces.
 */
const getIgnoredPackages = () => Object.freeze(ignoredPackages.slice(0));
exports.getIgnoredPackages = getIgnoredPackages;
const buildIgnoredPackages = () => {
    // just a safety precaution, no reason to ever do this
    /* c8 ignore start */
    if (!ignoredPackages.length)
        return undefined;
    /* c8 ignore stop */
    const p = ignoredPackages.map(s => regExpEscape(s));
    return new RegExp(`[/\\\\]node_modules[/\\\\](?:${p.join('|')})([/\\\\]|$)`);
};
let ignoredPackagesRE = buildIgnoredPackages();
let filterIgnoredPackages = true;
/**
 * Set whether or not the list of ignored packages should
 * be excluded from stack traces.
 */
const setFilterIgnoredPackages = (s) => (filterIgnoredPackages = s);
exports.setFilterIgnoredPackages = setFilterIgnoredPackages;
/**
 * Get whether or not the list of ignored packages should
 * be excluded from stack traces.
 */
const getFilterIgnoredPackages = () => filterIgnoredPackages;
exports.getFilterIgnoredPackages = getFilterIgnoredPackages;
// detect the first line of Error.stack, 'Error: blah'
const isErrorStackHead = (c) => !!c &&
    c.lineNumber === null &&
    c.columnNumber === null &&
    c.this === undefined &&
    c.evalOrigin === undefined &&
    c.function === undefined &&
    c.typeName === null &&
    c.methodName === null &&
    typeof c.functionName === 'string' &&
    c.isEval === false &&
    c.isNative === false &&
    c.isToplevel === false &&
    c.isConstructor === false &&
    c.generated === undefined;
const filter = (c) => {
    const s = c.fileName;
    // technically this is possible, but super unlikely
    /* c8 ignore start */
    if (!s)
        return true;
    /* c8 ignore stop */
    if (dirty && filterIgnoredPackages) {
        ignoredPackagesRE = buildIgnoredPackages();
        dirty = false;
    }
    return ((!filterNodeInternals ||
        !(s.startsWith('node:') || module_1.builtinModules.includes(s))) &&
        (!filterIgnoredPackages || !ignoredPackagesRE?.test(s)));
};
const clean = (c) => {
    const filtered = c.filter(filter);
    while (isErrorStackHead(filtered[0]))
        filtered.shift();
    if (cwd !== undefined) {
        for (const c of filtered) {
            c.cwd = cwd;
        }
    }
    return filtered;
};
let capturing = false;
function capture(limit = 0, fn = capture) {
    // should be impossible to hit, but can if we have to look up
    // a sourcemap for a file we haven't seen before, and the filename
    // happens to come through as a url or relative path.
    /* c8 ignore start */
    if (capturing)
        return [];
    /* c8 ignore stop */
    capturing = true;
    if (typeof limit === 'function') {
        fn = limit;
        limit = 0;
    }
    const { prepareStackTrace, stackTraceLimit } = Error;
    Error.prepareStackTrace = call_site_like_js_1.CallSiteLike.prepareStackTrace;
    if (limit) {
        // we always get an extra few frames to account for internals
        // or proxy frames that might be filtered out of the top.
        Error.stackTraceLimit = limit + 10;
    }
    const obj = { stack: [] };
    Error.captureStackTrace(obj, fn);
    const { stack } = obj;
    Object.assign(Error, { prepareStackTrace, stackTraceLimit });
    const st = clean(stack);
    capturing = false;
    return limit === 0 ? st : st.slice(0, limit);
}
exports.capture = capture;
/**
 * Get the current callsite location, from the location specified
 * by `fn`.
 */
const at = (fn = exports.at) => {
    const [site] = capture(1, fn);
    return site;
};
exports.at = at;
function captureString(limit = Infinity, fn = captureString) {
    if (typeof limit === 'function') {
        fn = limit;
        limit = 0;
    }
    const stack = capture(limit, fn);
    return (limit === 0 ? stack : stack.slice(0, limit))
        .map(c => String(c) + '\n')
        .join('');
}
exports.captureString = captureString;
/**
 * Get an array of {@link CallSiteLike} objects corresponding to the
 * stack trace of the Error object provided.
 */
const captureError = (e) => clean(
// errors almost always have stacks
/* c8 ignore start */
(e.stack || '')
    /* c8 ignore stop */
    .trim()
    .split('\n')
    .slice(1)
    .filter(l => !!l)
    .map(line => new call_site_like_js_1.CallSiteLike(e, line)));
exports.captureError = captureError;
/**
 * Get a processed string stack corresponding to the
 * stack trace of the Error object provided.
 */
const captureErrorString = (e) => (0, exports.captureError)(e)
    .map(c => String(c) + '\n')
    .join('');
exports.captureErrorString = captureErrorString;
/**
 * Parse a stack string and return an array of CallSiteLike objects
 *
 * We use this to get the `at` diagnostic callsite when all we have is
 * a stack, either from a native Error object, or a stringified CallSiteLike
 * stack.
 */
const parseStack = (s) => clean(s
    .trimEnd()
    .split('\n')
    .filter(l => !!l.trim())
    .map(line => new call_site_like_js_1.CallSiteLike(null, line)));
exports.parseStack = parseStack;
//# sourceMappingURL=index.js.map